﻿using System.Runtime.InteropServices;

namespace OBSBlur.Window;

[StructLayout(LayoutKind.Sequential)]
public struct Rectangle
{
   public int Left, Top, Right, Bottom;

   public Rectangle(int left, int top, int right, int bottom)
   {
     Left = left;
     Top = top;
     Right = right;
     Bottom = bottom;
   }

   public Rectangle(System.Drawing.Rectangle r) : this(r.Left, r.Top, r.Right, r.Bottom) { }

   public int X
   { 
     get { return Left; }
     set { Right -= (Left - value); Left = value; }
   }

   public int Y
   {
     get { return Top; }
     set { Bottom -= (Top - value); Top = value; }
   }

   public int Height
   {
     get { return Bottom - Top; }
     set { Bottom = value + Top; }
   }

   public int Width
   {
     get { return Right - Left; }
     set { Right = value + Left; }
   }

   public System.Drawing.Point Location
   {
     get { return new System.Drawing.Point(Left, Top); }
     set { X = value.X; Y = value.Y; }
   }

   public System.Drawing.Size Size
   {
     get { return new System.Drawing.Size(Width, Height); }
     set { Width = value.Width; Height = value.Height; }
   }

   public static implicit operator System.Drawing.Rectangle(Rectangle r)
   {
     return new System.Drawing.Rectangle(r.Left, r.Top, r.Width, r.Height);
   }

   public static implicit operator Rectangle(System.Drawing.Rectangle r)
   {
     return new Rectangle(r);
   }

   public static bool operator ==(Rectangle r1, Rectangle r2)
   {
     return r1.Equals(r2);
   }

   public static bool operator !=(Rectangle r1, Rectangle r2)
   {
     return !r1.Equals(r2);
   }

   public bool Equals(Rectangle r)
   {
     return r.Left == Left && r.Top == Top && r.Right == Right && r.Bottom == Bottom;
   }

   public override bool Equals(object obj)
   {
     if (obj is Rectangle)
       return Equals((Rectangle)obj);
     else if (obj is System.Drawing.Rectangle)
       return Equals(new Rectangle((System.Drawing.Rectangle)obj));
     return false;
   }

   public override int GetHashCode()
   {
     return ((System.Drawing.Rectangle)this).GetHashCode();
   }

   public override string ToString()
   {
     return $"{{LTRB {Left,-6:####0},{Top,-6:####0},{Right,-6:####0},{Bottom,-6:####0}}}";
   }
}